package main_test

import (
	"context"
	"crypto/ecdsa"
	"fmt"

	//"encoding/hex"
	"math/big"
	"net"
	"testing"
	"time"

	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/protobuf/proto"
	"google.golang.org/protobuf/types/known/anypb"

	base "github.com/CaduceusMetaverseProtocol/metaprotocol/gen/proto/go/base/v1"
	ring "github.com/CaduceusMetaverseProtocol/metaprotocol/gen/proto/go/ring/v1"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/core/types"
	"github.com/ethereum/go-ethereum/crypto"
)

// func BenchmarkFib10(b *testing.B) {
// 	for n := 0; n < b.N; n++ {
// 		//Fib(10)
// 	}
// }

type RingServer struct {
	ring.UnimplementedRingServiceServer
}

func (*RingServer) SendRawTransaction(ctx context.Context, req *base.EthTx) (*ring.SendRawTransactionResponse, error) {

	return &ring.SendRawTransactionResponse{}, nil
}

func (*RingServer) SendTxAsAny(ctx context.Context, req *base.Transaction) (*ring.SendRawTransactionResponse, error) {

	return &ring.SendRawTransactionResponse{}, nil
}

func (*RingServer) SendTxAsBytes(ctx context.Context, req *base.TransactionBytes) (*ring.SendRawTransactionResponse, error) {

	return &ring.SendRawTransactionResponse{}, nil
}

func TestGrpcServer(t *testing.T) {

	//onceFunc := func() {

	lis, err := net.Listen("tcp", ":9006")
	if err != nil {
		t.Fatal(err)
	}

	s := grpc.NewServer()

	ring.RegisterRingServiceServer(s, &RingServer{})

	//go func() {
	err = s.Serve(lis)
	if err != nil {
		t.Fatal(err)
	}
	//}()
	//}

	//once.Do(onceFunc)

}

func pricedTransaction(to common.Address, nonce uint64, gaslimit uint64, gasprice *big.Int, key *ecdsa.PrivateKey) *types.Transaction {
	tx, _ := types.SignTx(types.NewTransaction(nonce, to, big.NewInt(100), gaslimit, gasprice, nil), types.HomesteadSigner{}, key)
	return tx
}

// go  test -v -run EthTx  -bench=.  -benchtime=3s
// go test -v -run TestGrpcServer  -timeout 0

//BenchmarkAny
// go  test -v -run Any  -bench=.  -benchtime=3s

func BenchmarkAny(b *testing.B) {
	b.ReportAllocs()

	b.RunParallel(func(pb *testing.PB) {

		for pb.Next() {

			conn, err := grpc.Dial("127.0.0.1:9006", grpc.WithTransportCredentials(insecure.NewCredentials()))
			if err != nil {
				b.Fatal(err)
			}
			defer conn.Close()

			c := ring.NewRingServiceClient(conn)

			ctx, cancel := context.WithTimeout(context.Background(), time.Second)
			defer cancel()

			local, _ := crypto.HexToECDSA("FD5CC6F5E7E2805E920AC5DC83D5AF1106F9C92F0C04F9D5E1FD4261B4B4464A")

			//publicKey := local.Public()
			//publicKeyECDSA, _ := publicKey.(*ecdsa.PublicKey)
			//fromAddress := crypto.PubkeyToAddress(*publicKeyECDSA)
			//b.Log(fromAddress)

			remote, _ := crypto.GenerateKey()

			tx := pricedTransaction(crypto.PubkeyToAddress(remote.PublicKey), 0, 100000, big.NewInt(1), local)

			inner := base.EthTxData{
				AccountNonce: tx.Nonce(),
				Price:        tx.GasPrice().Bytes(),
				GasLimit:     tx.Gas(),
				Payload:      tx.Data(),
			}

			v, r, sigs := tx.RawSignatureValues()

			inner.V = v.Bytes()
			inner.R = r.Bytes()
			inner.S = sigs.Bytes()
			inner.Amount = tx.Value().Bytes()

			addr := base.Address{Address: tx.To().Bytes()}

			inner.Recipient = &addr

			// Each goroutine has its own bytes.Buffer.

			// The loop body is executed b.N times total across all goroutines.

			ethTx := base.EthTx{Inner: &inner}

			ethTxAsAny, err := pbany(ethTx)

			c.SendTxAsAny(ctx, &base.Transaction{Tx: ethTxAsAny})

			res, err := c.SendRawTransaction(ctx, &base.EthTx{Inner: &inner})

			if err != nil {
				b.Fatal(err)
			}

			if err != nil {
				b.Fatal(err)
			}

			_ = res

			//b.Logf("Greeting: %v\n", res)

		}
	})
}

func pbany(v interface{}) (*anypb.Any, error) {
	pv, ok := v.(proto.Message)
	if !ok {
		return &anypb.Any{}, fmt.Errorf("%v is not proto.Message", pv)
	}
	return anypb.New(pv)
}

func BenchmarkEthTx(b *testing.B) {
	b.ReportAllocs()

	b.RunParallel(func(pb *testing.PB) {

		for pb.Next() {

			conn, err := grpc.Dial("127.0.0.1:9006", grpc.WithTransportCredentials(insecure.NewCredentials()))
			if err != nil {
				b.Fatal(err)
			}
			defer conn.Close()

			c := ring.NewRingServiceClient(conn)

			ctx, cancel := context.WithTimeout(context.Background(), time.Second)
			defer cancel()

			local, _ := crypto.HexToECDSA("FD5CC6F5E7E2805E920AC5DC83D5AF1106F9C92F0C04F9D5E1FD4261B4B4464A")

			//publicKey := local.Public()
			//publicKeyECDSA, _ := publicKey.(*ecdsa.PublicKey)
			//fromAddress := crypto.PubkeyToAddress(*publicKeyECDSA)
			//b.Log(fromAddress)

			remote, _ := crypto.GenerateKey()

			tx := pricedTransaction(crypto.PubkeyToAddress(remote.PublicKey), 0, 100000, big.NewInt(1), local)

			inner := base.EthTxData{
				AccountNonce: tx.Nonce(),
				Price:        tx.GasPrice().Bytes(),
				GasLimit:     tx.Gas(),
				Payload:      tx.Data(),
			}

			v, r, sigs := tx.RawSignatureValues()

			inner.V = v.Bytes()
			inner.R = r.Bytes()
			inner.S = sigs.Bytes()
			inner.Amount = tx.Value().Bytes()

			addr := base.Address{Address: tx.To().Bytes()}

			inner.Recipient = &addr

			// Each goroutine has its own bytes.Buffer.

			// The loop body is executed b.N times total across all goroutines.

			res, err := c.SendRawTransaction(ctx, &base.EthTx{Inner: &inner})

			if err != nil {
				b.Fatal(err)
			}

			_ = res

			//b.Logf("Greeting: %v\n", res)

		}
	})
}

func BenchmarkBytes(b *testing.B) {
	b.ReportAllocs()

	b.RunParallel(func(pb *testing.PB) {

		for pb.Next() {

			conn, err := grpc.Dial("127.0.0.1:9006", grpc.WithTransportCredentials(insecure.NewCredentials()))
			if err != nil {
				b.Fatal(err)
			}
			defer conn.Close()

			c := ring.NewRingServiceClient(conn)

			ctx, cancel := context.WithTimeout(context.Background(), time.Second)
			defer cancel()

			local, _ := crypto.HexToECDSA("FD5CC6F5E7E2805E920AC5DC83D5AF1106F9C92F0C04F9D5E1FD4261B4B4464A")

			//publicKey := local.Public()
			//publicKeyECDSA, _ := publicKey.(*ecdsa.PublicKey)
			//fromAddress := crypto.PubkeyToAddress(*publicKeyECDSA)
			//b.Log(fromAddress)

			remote, _ := crypto.GenerateKey()

			tx := pricedTransaction(crypto.PubkeyToAddress(remote.PublicKey), 0, 100000, big.NewInt(1), local)

			// inner := base.EthTxData{
			// 	AccountNonce: tx.Nonce(),
			// 	Price:        tx.GasPrice().Bytes(),
			// 	GasLimit:     tx.Gas(),
			// 	Payload:      tx.Data(),
			// }

			// v, r, sigs := tx.RawSignatureValues()

			// inner.V = v.Bytes()
			// inner.R = r.Bytes()
			// inner.S = sigs.Bytes()
			// inner.Amount = tx.Value().Bytes()

			// addr := base.Address{Address: tx.To().Bytes()}

			// inner.Recipient = &addr

			// Each goroutine has its own bytes.Buffer.

			// The loop body is executed b.N times total across all goroutines.

			//res, err := c.SendRawTransaction(ctx, &base.EthTx{Inner: &inner})

			txAsBytes, err := tx.MarshalBinary()

			if err != nil {
				b.Fatal(err)
			}

			res, err := c.SendTxAsBytes(ctx, &base.TransactionBytes{Tx: txAsBytes})

			if err != nil {
				b.Fatal(err)
			}

			_ = res

			//b.Logf("Greeting: %v\n", res)

		}
	})
}
